import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';
import 'react-virtualized/styles.css';

import { COMPONENTS } from 'common/ui-labels';

import TableButtons from './TableButtons';
import withSortSelect from '../../../../src/js/components/HOC/withSortSelect';
import { formatDbNameToHumanReadable, virtualizedHeaderRenderer } from '../../../../src/js/utils/htmlUtils';
import { ZONE_DOWNLOAD_STATUS } from '../../common/constants';
import { isInProgress } from '../../utils';


class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trStyleFormat: PropTypes.func,
        setFilterItems: PropTypes.func,
        handleSelectRow: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        filteredItems: PropTypes.arrayOf(PropTypes.shape()),
    
        // Props from parent
        allItems: PropTypes.shape(),
        tableSize: PropTypes.shape(),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
        dispatch: PropTypes.func,
        checkboxButton: PropTypes.func,
        headerCheckbox: PropTypes.func,
        disableActions: PropTypes.bool,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trStyleFormat: () => { },
        setFilterItems: () => { },
        handleSelectRow: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        filteredItems: [],
    
        // Props from parent
        allItems: new Map(),
        tableSize: {},
        selectedItems: [],
        dispatch: () => { },
        checkboxButton: () => { },
        headerCheckbox: () => { },
        disableActions: false,
        intl: {
            formatMessage: () => {},
        },
    };
    
    constructor(props) {
        super(props);
        this.dispatch = props.dispatch;
        
        // HOC Methods
        this.handleSelectRow = props.handleSelectRow.bind(this);
        this.checkboxButton = props.checkboxButton.bind(this);
        this.headerCheckbox = props.headerCheckbox.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
        this.setFilterItems = props.setFilterItems.bind(this);
    }

    trStyleFormat = () => ({ alignItems: 'stretch' });
    
    trClassFormat = (items, { index }) => {
        const currentItem = items[index];
        if (index >= 0 && currentItem) {
            if (isInProgress(currentItem.download_status)) {
                return 'Table-InProgress-Row';
            }
            if (!currentItem.isResponsive) {
                return 'Table-Unresponsive-Row';
            }
            switch (currentItem.download_status) {
                case ZONE_DOWNLOAD_STATUS.SYSTEM.OOSYNC: {
                    return 'Table-OutOfSync-Row';
                }
                case ZONE_DOWNLOAD_STATUS.SYSTEM.OUTDATED: {
                    return 'Table-Outdated-Row';
                }
                case ZONE_DOWNLOAD_STATUS.SYSTEM.UPDATED: {
                    return 'Table-Updated-Row';
                }
                default: {
                    return 'Table-Unresponsive-Row';
                }
            }
        }
        return 'Virtual-Table-Header-Row';
    };
    
    render() {
        const { sortBy, sortDirection, tableSize, allItems, selectedItems, filteredItems, disableActions, intl } = this.props;
        const allItemsArray = [...allItems.values()];
        const displayItems = this.sortDisplayItems(filteredItems);
        const displayCount = displayItems.length;
        const columnWidth = Math.round((tableSize.width - 100) / 3);
    
        return (
            <section>
                <TableButtons
                    items={allItemsArray}
                    width={tableSize.width}
                    setFilterItems={this.setFilterItems}
                    selectedItems={selectedItems}
                    disableActions={disableActions}
                    searchFields={['mac_address', 'name', 'download_status_display']}
                />
                <Table
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Medium"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header react-bs-container-header"
                    rowClassName={({ index }) => this.trClassFormat(displayItems, { index })}
                    rowStyle={({ index }) => this.trStyleFormat()}
                    onRowClick={({ event, index, rowData }) => this.handleSelectRow(displayCount, { event, index, rowData })}
                    rowCount={displayCount} // required
                    rowGetter={({ index }) => displayItems[index]} // required
                >
                    <Column
                        label={<span title="">{this.headerCheckbox(displayItems)}</span>}
                        dataKey="checkbox"
                        cellRenderer={({ rowData }) => this.checkboxButton(rowData, displayCount)}
                        headerRenderer={virtualizedHeaderRenderer}
                        width={100}
                        disableSort
                        headerClassName="fixed-width"
                        className="Virtual-Table-Column-Checkbox first-column fixed-width"
                    />

                    <Column
                        width={columnWidth}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.TRACKABLE}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="name"
                        id="vehicle-table-name"
                        className="Virtual-Table-Column-Center"
                    />
                    
                    <Column
                        width={columnWidth}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.MAC_ADDRESS}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="mac_address"
                        id="vehicle-table-mac"
                        className="Virtual-Table-Column-Center"
                    />
                    
                    <Column
                        width={columnWidth}
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.STATUS}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData, rowData }) => {
                            if (cellData === ZONE_DOWNLOAD_STATUS.DISPLAY.UNRESPONSIVE) {
                                return formatDbNameToHumanReadable(intl, ZONE_DOWNLOAD_STATUS.DISPLAY.UNRESPONSIVE, cellData);
                            }
                            return formatDbNameToHumanReadable(intl, rowData.download_status, cellData);
                        }}
                        dataKey="download_status_display"
                        id="vehicle-table-status"
                        className="Virtual-Table-Column-Center"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
