import { getSyncDateByTz, getTimeZoneOfProject } from 'utils/timeUtils';
import _isEmpty from 'lodash/isEmpty';
import _Get from 'lodash/get';
import Trackable from 'stores/classes/Trackable';
import { MODULE, ZONE_DOWNLOAD_STATUS } from './../common/constants';


export const isOutOfSync = status => status === ZONE_DOWNLOAD_STATUS.SYSTEM.OOSYNC;
export const isOutdated = status => status === ZONE_DOWNLOAD_STATUS.SYSTEM.OUTDATED;
export const isInProgress = status => status === ZONE_DOWNLOAD_STATUS.SYSTEM.IN_PROCESS;
export const isUpdated = status => status === ZONE_DOWNLOAD_STATUS.SYSTEM.UPDATED;

export const defineDownloadStatus = (status) => {
    if (isInProgress(status.download_status) || status.sync_requested) {
        return ZONE_DOWNLOAD_STATUS.SYSTEM.IN_PROCESS;
    }
    return status.download_status;
};

export const mapIterationByCondition = (itemsMap, condition) => {
    let result = false;
    const mapIterator = itemsMap.values();
    
    while (true) {
        const iterator = mapIterator.next();
        const value = iterator.value;
        if (iterator.done || result) {
            break;
        }
        result = condition(value);
    }
    return result;
};

export const hasInProgressStatus = (trackables) => {
    const condition = trackable => (trackable.download_status === ZONE_DOWNLOAD_STATUS.SYSTEM.IN_PROCESS);
    return mapIterationByCondition(trackables, condition);
};

const getStatuses = (state) => {
    if (state[MODULE.ZONE_MANAGER]) {
        return Object.values(state[MODULE.ZONE_MANAGER].zoneDownloadStatus.byUid);
    }
    return [];
};

const getDisplayStatus = (trackable) => {
    if (isInProgress(trackable.download_status)) {
        return ZONE_DOWNLOAD_STATUS.DISPLAY.IN_PROCESS;
    }
    
    if (!trackable.isResponsive) {
        return ZONE_DOWNLOAD_STATUS.DISPLAY.UNRESPONSIVE;
    }
    
    switch (trackable.download_status) {
        case ZONE_DOWNLOAD_STATUS.SYSTEM.OOSYNC: {
            return ZONE_DOWNLOAD_STATUS.DISPLAY.OOSYNC;
        }
        case ZONE_DOWNLOAD_STATUS.SYSTEM.OUTDATED: {
            return ZONE_DOWNLOAD_STATUS.DISPLAY.OUTDATED;
        }
        case ZONE_DOWNLOAD_STATUS.SYSTEM.UPDATED: {
            return ZONE_DOWNLOAD_STATUS.DISPLAY.UPDATED;
        }
        default: {
            return ZONE_DOWNLOAD_STATUS.DISPLAY.UNRESPONSIVE;
        }
    }
};

const getLastTransaction = (state) => {
    const zoneManagerState = state[MODULE.ZONE_MANAGER];
    if (zoneManagerState) {
        return zoneManagerState.zoneDownloadStatus.lastSyncTransaction;
    }
    return undefined;
};

export const getLastSyncDate = (state) => {
    const lastTransaction = getLastTransaction(state);
    if (lastTransaction) {
        const timezone = getTimeZoneOfProject(state.appContext.project);
        return _isEmpty(lastTransaction) ? '-' : getSyncDateByTz(lastTransaction.created_at, timezone);
    }
    return '-';
};

export const getLastSyncNumber = (state) => {
    const lastTransaction = getLastTransaction(state);
    const trackable_uids = _Get(lastTransaction, 'trackable_uids', undefined);
    if (trackable_uids) {
        return (trackable_uids.split(',')).length;
    }
    return 0;
};

export const getFormattedTrackables = (state) => {
    const trackables = Trackable.getAssociatedTrackables(state);
    const statuses = getStatuses(state);
    const tkoMap = new Map();
    
    trackables.forEach((_trackable) => {
        const status = statuses.find(download_status => _trackable.uid === download_status.trackable_uid);
        
        if (status && _trackable.node) {
            const trackable = _trackable.getProperties();
            trackable.download_status = defineDownloadStatus(status);
            trackable.last_synced = status.ts;
            trackable.mac_address = trackable.node.mac_address;
            trackable.isResponsive = _trackable.isResponsive(state.playback.isOpenPlayBackControl);
            trackable.download_status_display = getDisplayStatus(trackable);
            
            tkoMap.set(trackable.uid, trackable);
        }
    });
    
    return tkoMap;
};

export const defineStatusCount = (trackables) => {
    const arrayTrackablesList = [...trackables.values()];
    const statusCount = {
        total: trackables.size,
        outOfSync: 0,
        outDated: 0,
        inProgress: 0,
        updated: 0,
        unresponsive: 0,
    };
    
    arrayTrackablesList.forEach((tko) => {
        if (tko.isResponsive || isInProgress(tko.download_status)) {
            statusCount.outOfSync += isOutOfSync(tko.download_status) ? 1 : 0;
            statusCount.outDated += isOutdated(tko.download_status) ? 1 : 0;
            statusCount.inProgress += isInProgress(tko.download_status) ? 1 : 0;
            statusCount.updated += isUpdated(tko.download_status) ? 1 : 0;
        } else {
            statusCount.unresponsive += 1;
        }
    });
    
    return statusCount;
};
