const chai = require('chai');
const sinon = require('sinon');
const chaiAsPromised = require('chai-as-promised');

const { ZoneDownloadRequestRepo } = require('../../repository/zone_download_request');
const { ZMQ_OBJECT, TYPE_ACTION } = require('enums').OBJECT_TYPE;
const ZMQUtils = require('./../../../../routes/utils_zmq');
const Service = require('../service');

chai.use(chaiAsPromised);
const { expect } = chai;


let ZoneDownloadRequestRepo_insertStatus;
let ZoneDownloadRequestRepo_updateStatus;
let ZoneDownloadRequestRepo_getStatuses;
let ZoneDownloadRequestRepo_getTransaction;
let ZoneDownloadRequestRepo_getValidTrackables;
let ZMQUtils_notify;


describe('zone_download_request service', () => {
    beforeEach(() => {
        ZoneDownloadRequestRepo_insertStatus = sinon.stub(ZoneDownloadRequestRepo, 'insertStatus');
        ZoneDownloadRequestRepo_updateStatus = sinon.stub(ZoneDownloadRequestRepo, 'updateStatus');
        ZoneDownloadRequestRepo_getStatuses = sinon.stub(ZoneDownloadRequestRepo, 'getStatuses');
        ZoneDownloadRequestRepo_getTransaction = sinon.stub(ZoneDownloadRequestRepo, 'getTransaction');
        ZoneDownloadRequestRepo_getValidTrackables = sinon.stub(ZoneDownloadRequestRepo, 'getValidTrackables');
        ZMQUtils_notify = sinon.stub(ZMQUtils, 'notify');
    });
    
    afterEach(() => {
        ZoneDownloadRequestRepo_insertStatus.restore();
        ZoneDownloadRequestRepo_updateStatus.restore();
        ZoneDownloadRequestRepo_getStatuses.restore();
        ZoneDownloadRequestRepo_getTransaction.restore();
        ZoneDownloadRequestRepo_getValidTrackables.restore();
        ZMQUtils_notify.restore();
    });
    
    describe('handleDownloadRequestUpdate', () => {
        it('it should save transaction and send notify', async () => {
            const requestObject = { scope: { projectTag: { uid: 'project_uid' } } };
            const trackables = [{ trackable_uid: 'trackable_uid_one' }];
            const plist = { zone_download_request: trackables };
    
            const transaction = {
                trackable_uids: trackables.map(tko => tko.trackable_uid).join(','),
                project_uid: requestObject.scope.projectTag.uid,
            };
            const transaction_uids = trackables.map(tko => tko.trackable_uid);

            const statuses = trackables.map(tko => ({ trackable_uid: tko.trackable_uid, download_status: 'OOSYNC', sync_requested: 0 }));
            
            ZoneDownloadRequestRepo_getStatuses.returns(statuses);

            await Service.handleDownloadRequestUpdate(requestObject, trackables, plist);

            expect(ZoneDownloadRequestRepo_insertStatus.callCount).to.equal(1);
            expect(ZoneDownloadRequestRepo_updateStatus.withArgs(true, transaction_uids).callCount).to.equal(1);
            expect(ZoneDownloadRequestRepo_getStatuses.withArgs(transaction_uids).callCount).to.equal(1);

            expect(ZMQUtils_notify.callCount).to.equal(3);
            expect(ZMQUtils_notify.withArgs({
                req: sinon.match.any,
                object: ZMQ_OBJECT.ZONE_DOWNLOAD_TRANSACTION,
                action: TYPE_ACTION.CREATED,
                plist: {
                    transaction: {
                        trackable_uids: transaction.trackable_uids,
                        project_uid: transaction.project_uid,
                        created_at: sinon.match.any,
                        uid: sinon.match.string,
                    },
                },
            }).callCount).to.deep.equal(1);
            expect(ZMQUtils_notify.withArgs({
                req: sinon.match.any,
                object: ZMQ_OBJECT.ZONE_DOWNLOAD_STATUS,
                action: TYPE_ACTION.CREATED,
                plist: statuses,
            }).callCount).to.deep.equal(1);
            expect(ZMQUtils_notify.withArgs({
                req: sinon.match.any,
                object: ZMQ_OBJECT.ZONE_DOWNLOAD_REQUEST,
                action: TYPE_ACTION.UPDATE,
                plist,
            }).callCount).to.deep.equal(1);
        });
    });
    
    describe('handleDownloadRequestVerify', () => {
        it('it should send verification notify', async () => {
            const requestObject = { scope: { projectTag: { uid: 'project_uid' } } };
            const trackables = [{ trackable_uid: 'trackable_uid_one' }];
            const plist = { zone_download_request: trackables };
            await Service.handleDownloadRequestVerify(requestObject, plist);

            expect(ZMQUtils_notify.callCount).to.equal(1);
            expect(ZMQUtils_notify.withArgs({
                req: sinon.match.any,
                object: ZMQ_OBJECT.ZONE_DOWNLOAD_REQUEST,
                action: TYPE_ACTION.VERIFY,
                plist,
            }).callCount).to.deep.equal(1);
        });
    });
});
