const chai = require('chai');
const sinon = require('sinon');
const chaiAsPromised = require('chai-as-promised');

const { ZoneDownloadStatusRepo } = require('../../repository/zone_download_status');
const DownloadStatus = require('../index');

chai.use(chaiAsPromised);
const { expect } = chai;


let ZoneDownloadStatusRepo_getTrackablesProjects;
let ZoneDownloadStatusRepo_getZoneDownloadStatus;
let ZoneDownloadStatusRepo_getTrackableWithProject;
let ZoneDownloadStatusRepo_deleteStatusByTrackable;
let ZoneDownloadStatusRepo_upsertStatus;


describe('zone_download_status service', () => {
    beforeEach(() => {
        ZoneDownloadStatusRepo_getTrackablesProjects = sinon.stub(ZoneDownloadStatusRepo, 'getTrackablesProjects');
        ZoneDownloadStatusRepo_getZoneDownloadStatus = sinon.stub(ZoneDownloadStatusRepo, 'getZoneDownloadStatus');
        ZoneDownloadStatusRepo_getTrackableWithProject = sinon.stub(ZoneDownloadStatusRepo, 'getTrackableWithProject');
        ZoneDownloadStatusRepo_deleteStatusByTrackable = sinon.stub(ZoneDownloadStatusRepo, 'deleteStatusByTrackable');
        ZoneDownloadStatusRepo_upsertStatus = sinon.stub(ZoneDownloadStatusRepo, 'upsertStatus');
    });
    
    afterEach(() => {
        ZoneDownloadStatusRepo_getTrackablesProjects.restore();
        ZoneDownloadStatusRepo_getZoneDownloadStatus.restore();
        ZoneDownloadStatusRepo_getTrackableWithProject.restore();
        ZoneDownloadStatusRepo_deleteStatusByTrackable.restore();
        ZoneDownloadStatusRepo_upsertStatus.restore();
    });
    
    describe('getProjectStatusMap', () => {
        it('it should return empty Map when called with empty arguments', async () => {
            const emptyMap = new Map();
            const firstCaseMap = DownloadStatus.getProjectStatusMap();
            const secondCaseMap = DownloadStatus.getProjectStatusMap([], []);
            const thirdCaseMap = DownloadStatus.getProjectStatusMap(undefined, null);
            expect(firstCaseMap).to.deep.equal(emptyMap);
            expect(secondCaseMap).to.deep.equal(emptyMap);
            expect(thirdCaseMap).to.deep.equal(emptyMap);
        });
        
        it('it should return Map with project keys for two trackables', async () => {
            const project_one = { uid: 'project_uid_one' };
            const project_two = { uid: 'project_uid_two' };
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            
            const status_values = [{ trackable_uid: trackable_one.uid, status: 'OOSYNC' }, { trackable_uid: trackable_two.uid, status: 'OOSYNC' }];
            const trackables_projects = [{ project_uid: project_one.uid, uid: trackable_one.uid }, { project_uid: project_two.uid, uid: trackable_two.uid }];
            
            const projectMap = DownloadStatus.getProjectStatusMap(status_values, trackables_projects);
            expect(projectMap.size).to.equal(2);
            expect(projectMap.get(project_one.uid).length).to.equal(1);
            expect(projectMap.get(project_two.uid).length).to.equal(1);
        });
    
        it('it should return Map with project keys for three trackables', async () => {
            const project_one = { uid: 'project_uid_one' };
            const project_two = { uid: 'project_uid_two' };
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            const trackable_three = { uid: 'trackable_uid_3' };
        
            const status_values = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC' },
                { trackable_uid: trackable_two.uid, download_status: 'OOSYNC' },
                { trackable_uid: trackable_three.uid, download_status: 'OOSYNC' },
            ];
            const trackables_projects = [
                { project_uid: project_one.uid, uid: trackable_one.uid },
                { project_uid: project_two.uid, uid: trackable_two.uid },
                { project_uid: project_two.uid, uid: trackable_three.uid },
            ];
        
            const projectMap = DownloadStatus.getProjectStatusMap(status_values, trackables_projects);
            expect(projectMap.size).to.equal(2);
            expect(projectMap.get(project_one.uid).length).to.equal(1);
            expect(projectMap.get(project_two.uid).length).to.equal(2);
        });
    });
    
    describe('getUniqueStatusList', () => {
        it('it should return empty lists when called with empty arguments', async () => {
            const emptyResult = DownloadStatus.getUniqueStatusList();
            
            expect(emptyResult.status_list).to.deep.equal([]);
            expect(emptyResult.trackable_uids).to.deep.equal([]);
        });
        
        it('it should return empty lists when called with empty arguments', async () => {
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            const trackable_three = { uid: 'trackable_uid_3' };
    
            const statuses = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC' },
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC' },
                { trackable_uid: trackable_two.uid, download_status: 'OOSYNC' },
                { trackable_uid: trackable_two.uid, download_status: 'OUTDATED' },
                { trackable_uid: trackable_three.uid, download_status: 'OOSYNC' },
                { trackable_uid: trackable_three.uid, download_status: 'IN_PROGRESS' },
            ];
            
            const filtered_statuses = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC' },
                { trackable_uid: trackable_two.uid, download_status: 'OUTDATED' },
                { trackable_uid: trackable_three.uid, download_status: 'IN_PROGRESS' },
            ];
            
            const emptyResult = DownloadStatus.getUniqueStatusList(statuses);
            
            expect(emptyResult.status_list).to.deep.equal(filtered_statuses);
            expect(emptyResult.trackable_uids).to.deep.equal([trackable_one.uid, trackable_two.uid, trackable_three.uid]);
        });
    });
    
    describe('getDataForProjects', () => {
        it('it should get formatted status project data', async () => {
            const project_one = { uid: 'project_uid_one' };
            const project_two = { uid: 'project_uid_two' };
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            const trackable_three = { uid: 'trackable_uid_3' };
        
            const trackable_uids = [trackable_one.uid, trackable_two.uid, trackable_three.uid];
            const status_list = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC', reason: 'Some Reason' },
                { trackable_uid: trackable_two.uid, download_status: 'OUTDATED', reason: 'Some Reason' },
                { trackable_uid: trackable_three.uid, download_status: 'IN_PROGRESS', reason: 'Some Reason' },
            ];
            const _trackables_projects = [
                { project_uid: project_one.uid, uid: trackable_one.uid },
                { project_uid: project_two.uid, uid: trackable_two.uid },
                { project_uid: project_two.uid, uid: trackable_three.uid },
            ];

            ZoneDownloadStatusRepo_getTrackablesProjects.returns(_trackables_projects);

            const { formatted_status_list, filtered_status_values, trackables_projects } = await DownloadStatus.getDataForProjects(trackable_uids, status_list);

            expect(ZoneDownloadStatusRepo_getTrackablesProjects.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_getZoneDownloadStatus.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getTrackableWithProject.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_deleteStatusByTrackable.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_upsertStatus.callCount).to.be.equal(0);

            expect(formatted_status_list).to.deep.equal(status_list.map(status => ([status.trackable_uid, status.download_status, status.reason])));
            expect(filtered_status_values).to.deep.equal(status_list);
            expect(trackables_projects).to.deep.equal(_trackables_projects);
        });
    
        it('it should filter status for trackables without project', async () => {
            const project_one = { uid: 'project_uid_one' };
            const project_two = { uid: 'project_uid_two' };
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            const trackable_three = { uid: 'trackable_uid_3' };
        
            const trackable_uids = [trackable_one.uid, trackable_two.uid, trackable_three.uid];
            const status_list = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC', reason: 'Some Reason' },
                { trackable_uid: trackable_two.uid, download_status: 'OUTDATED', reason: 'Some Reason' },
                { trackable_uid: trackable_three.uid, download_status: 'IN_PROGRESS', reason: 'Some Reason' },
            ];
        
            const status_list_for_two_tko = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC', reason: 'Some Reason' },
                { trackable_uid: trackable_two.uid, download_status: 'OUTDATED', reason: 'Some Reason' },
            ];
        
            const _trackables_projects = [
                { project_uid: project_one.uid, uid: trackable_one.uid },
                { project_uid: project_two.uid, uid: trackable_two.uid },
            ];
        
            ZoneDownloadStatusRepo_getTrackablesProjects.returns(_trackables_projects);

            const { formatted_status_list, filtered_status_values, trackables_projects } = await DownloadStatus.getDataForProjects(trackable_uids, status_list);

            expect(ZoneDownloadStatusRepo_getTrackablesProjects.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_getZoneDownloadStatus.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getTrackableWithProject.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_deleteStatusByTrackable.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_upsertStatus.callCount).to.be.equal(0);

            expect(formatted_status_list).to.deep.equal(status_list_for_two_tko.map(status => ([status.trackable_uid, status.download_status, status.reason])));
            expect(filtered_status_values).to.deep.equal(status_list_for_two_tko);
            expect(trackables_projects).to.deep.equal(_trackables_projects);
        });
    
        it('it should return empty values if any argument is empty 1', async () => {
            const project_one = { uid: 'project_uid_one' };
            const project_two = { uid: 'project_uid_two' };
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            const trackable_three = { uid: 'trackable_uid_3' };
        
            const trackable_uids = [trackable_one.uid, trackable_two.uid, trackable_three.uid];
            const status_list = [];
        
            const _trackables_projects = [
                { project_uid: project_one.uid, uid: trackable_one.uid },
                { project_uid: project_two.uid, uid: trackable_two.uid },
            ];
        
            ZoneDownloadStatusRepo_getTrackablesProjects.returns(_trackables_projects);

            const { formatted_status_list, filtered_status_values, trackables_projects } = await DownloadStatus.getDataForProjects(trackable_uids, status_list);

            expect(ZoneDownloadStatusRepo_getTrackablesProjects.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_getZoneDownloadStatus.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getTrackableWithProject.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_deleteStatusByTrackable.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_upsertStatus.callCount).to.be.equal(0);

            expect(formatted_status_list).to.deep.equal([]);
            expect(filtered_status_values).to.deep.equal([]);
            expect(trackables_projects).to.deep.equal(_trackables_projects);
        });
    
        it('it should return empty values if any argument is empty 2', async () => {
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            const trackable_three = { uid: 'trackable_uid_3' };
        
            const trackable_uids = [trackable_one.uid, trackable_two.uid, trackable_three.uid];
            const status_list = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC', reason: 'Some Reason' },
                { trackable_uid: trackable_two.uid, download_status: 'OUTDATED', reason: 'Some Reason' },
                { trackable_uid: trackable_three.uid, download_status: 'IN_PROGRESS', reason: 'Some Reason' },
            ];
        
            const _trackables_projects = [];
        
            ZoneDownloadStatusRepo_getTrackablesProjects.returns(_trackables_projects);

            const { formatted_status_list, filtered_status_values, trackables_projects } = await DownloadStatus.getDataForProjects(trackable_uids, status_list);

            expect(ZoneDownloadStatusRepo_getTrackablesProjects.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_getZoneDownloadStatus.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getTrackableWithProject.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_deleteStatusByTrackable.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_upsertStatus.callCount).to.be.equal(0);

            expect(formatted_status_list).to.deep.equal([]);
            expect(filtered_status_values).to.deep.equal([]);
            expect(trackables_projects).to.deep.equal([]);
        });
    });
    
    describe('removeZoneDownloadStatus', () => {
        it('it should call remove sql if find the trackable by status', async () => {
            const trackable_one = { uid: 'trackable_uid_1', project_uid: 'project_uid_1' };
            ZoneDownloadStatusRepo_getTrackableWithProject.returns(trackable_one);
            
            await DownloadStatus.removeZoneDownloadStatus(trackable_one.uid);

            expect(ZoneDownloadStatusRepo_getTrackablesProjects.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getZoneDownloadStatus.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getTrackableWithProject.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_deleteStatusByTrackable.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_upsertStatus.callCount).to.be.equal(0);
        });
    
        it('it should not call remove sql if find the trackable by status', async () => {
            const trackable_one = { uid: 'trackable_uid_1', project_uid: 'project_uid_1' };

            ZoneDownloadStatusRepo_getTrackableWithProject.returns(undefined);

            await DownloadStatus.removeZoneDownloadStatus(trackable_one.uid);

            expect(ZoneDownloadStatusRepo_getTrackablesProjects.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getZoneDownloadStatus.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getTrackableWithProject.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_deleteStatusByTrackable.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_upsertStatus.callCount).to.be.equal(0);
        });
    });
    
    describe('setZoneDownloadStatus', () => {
        it('it should return empty Map for empty arguments', async () => {
            const firstProjectStatusMap = await DownloadStatus.setZoneDownloadStatus();
            const secondProjectStatusMap = await DownloadStatus.setZoneDownloadStatus([]);
            const thirdProjectStatusMap = await DownloadStatus.setZoneDownloadStatus([]);

            expect(firstProjectStatusMap).to.deep.equal(new Map());
            expect(secondProjectStatusMap).to.deep.equal(new Map());
            expect(thirdProjectStatusMap).to.deep.equal(new Map());
        });

        it('it should return empty Map if not found trackables projects', async () => {
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            const trackable_three = { uid: 'trackable_uid_3' };

            const zone_download_statuses = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC', reason: 'Some Reason' },
                { trackable_uid: trackable_two.uid, download_status: 'OUTDATED', reason: 'Some Reason' },
                { trackable_uid: trackable_three.uid, download_status: 'IN_PROGRESS', reason: 'Some Reason' },
            ];

            ZoneDownloadStatusRepo_getTrackablesProjects.returns([]);

            const projectStatusMap = await DownloadStatus.setZoneDownloadStatus(zone_download_statuses);
            expect(projectStatusMap).to.deep.equal(new Map());

            expect(ZoneDownloadStatusRepo_getTrackablesProjects.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_getZoneDownloadStatus.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getTrackableWithProject.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_deleteStatusByTrackable.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_upsertStatus.callCount).to.be.equal(0);
        });
    
        it('it should return projects Map', async () => {
            const project_one = { uid: 'project_uid_1' };
            const project_two = { uid: 'project_uid_2' };
            const trackable_one = { uid: 'trackable_uid_1' };
            const trackable_two = { uid: 'trackable_uid_2' };
            const trackable_three = { uid: 'trackable_uid_3' };
        
            const zone_download_statuses = [
                { trackable_uid: trackable_one.uid, download_status: 'OOSYNC', reason: 'Some Reason' },
                { trackable_uid: trackable_two.uid, download_status: 'OUTDATED', reason: 'Some Reason' },
                { trackable_uid: trackable_three.uid, download_status: 'IN_PROGRESS', reason: 'Some Reason' },
            ];
            const trackables_projects = [
                { project_uid: project_one.uid, uid: trackable_one.uid },
                { project_uid: project_two.uid, uid: trackable_two.uid },
            ];

            ZoneDownloadStatusRepo_getTrackablesProjects.returns(trackables_projects);
            ZoneDownloadStatusRepo_upsertStatus.returns(Promise.resolve({}));

            const projectStatusMap = (await DownloadStatus.setZoneDownloadStatus(zone_download_statuses));

            expect(projectStatusMap.size).to.equal(2);
            expect(projectStatusMap.get(project_one.uid)).to.deep.equal([zone_download_statuses[0]]);
            expect(projectStatusMap.get(project_two.uid)).to.deep.equal([zone_download_statuses[1]]);

            expect(ZoneDownloadStatusRepo_getTrackablesProjects.callCount).to.be.equal(1);
            expect(ZoneDownloadStatusRepo_getZoneDownloadStatus.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_getTrackableWithProject.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_deleteStatusByTrackable.callCount).to.be.equal(0);
            expect(ZoneDownloadStatusRepo_upsertStatus.callCount).to.be.equal(1);
        });
    });
});
